
package biggests.gui;

import javax.swing.JOptionPane;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;
import java.io.File;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import java.awt.Component;
import javax.swing.JDialog;
import org.jfree.chart.ChartUtilities;
import javax.swing.JTextField;
import java.awt.Frame;
import biggests.utils.FileFilterExtension;
import com.lowagie.text.pdf.*;
import java.awt.Dimension;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.IOException;

/**
 * <p>Title: Save Chart Image Dialog</p>
 *
 * <p>Description: Dialog suited for prompting the user to provide the
 *                 width, height and file format to export a chart as
 *                 an image.</p>
 *
 * <p>Copyright:   Copyright (C) 2007  Joana Gonçalves
 *                 This program is free software; you can redistribute
 *                 it and/or modify it under the terms of the GNU General
 *                 Public License as published by the Free Software
 *                 Foundation; either version 3 of the License, or
 *                 any later version.
 *
 *                 This program is distributed in the hope that it will
 *                 be useful, but WITHOUT ANY WARRANTY; without even the
 *                 implied warranty of MERCHANTABILITY or FITNESS FOR A
 *                 PARTICULAR PURPOSE.  See the
 *                 <a href="http://www.gnu.org/licenses/gpl.html">
 *                 GNU General Public License</a> for more details.
 * </p>
 *
 * @author Joana P. Gonçalves
 */
public class SaveChartImageDialog extends JDialog {
    /**
     * PNG document file extension.
     */
    public static final String PNG = ".png";

    /**
     * SVG document file extension.
     */
    public static final String SVG = ".svg";

    /**
     * PDF document file extension.
     */
    public static final String PDF = ".pdf";

    /**
     * The width of the image.
     */
    private int imageWidth;

    /**
     * The height of the image.
     */
    private int imageHeight;

    /**
     * Component(s) to export to image file(s).
     */
    private Component chartPanel;

    /**
     * Image size label.
     */
    private JLabel imageSizeLabel;

    /**
     * File path label.
     */
    private JLabel fileLabel;

    /**
     * File path input text field.
     */
    private JTextField fileTextField;

    /**
     * Browse file path button.
     */
    private JButton browseButton;

    /**
     * Displayed image size radio button.
     */
//    private JRadioButton displayedSizeRadioButton;

    /**
     * Custom size radio button
     */
    private JRadioButton customSizeRadioButton;

    /**
     * Width label.
     */
    private JLabel widthLabel;

    /**
     * Width input text field.
     */
    private JTextField widthTextField;

    /**
     * Height label.
     */
    private JLabel heightLabel;

    /**
     * Height input text field.
     */
    private JTextField heightTextField;

    /**
     * Size button group.
     */
    //private ButtonGroup sizeButtonGroup;

    /**
     * Save button.
     */
    private JButton saveButton;

    /**
     * Cancel button.
     */
    private JButton cancelButton;

    /**
     * Creates a save chart image dialog.
     *
     * @param owner <code>Frame</code> the owner of the dialog
     * @param chartPanel the <code>Component</code> to be
     *        exported to image file
     * @param title <code>String</code> the title of the dialog box
     * @param modal <code>boolean</code> <code>true</code> if dialog
     *        box should be modal, <code>false</code> otherwise
     * @param imageWidth int the width of the image
     * @param imageHeight int the height of the image
     */
    public SaveChartImageDialog(Frame owner, Component chartPanel,
                                String title, boolean modal,
                                int imageWidth, int imageHeight) {
        super(owner, title, modal);
        setSize(300, 250);
        this.imageWidth = imageWidth;
        this.imageHeight = imageHeight;
        this.chartPanel = chartPanel;
        jbInit();
    }

    /**
     * Initializes GUI components.
     */
    public void jbInit() {
        JPanel contentPane = (JPanel)this.getContentPane();
        contentPane.setBorder(BorderFactory.createEmptyBorder(20, 20, 20,
                20));
        contentPane.setLayout(new SpringLayout());
        imageSizeLabel = new JLabel("Image size");
//        displayedSizeRadioButton = new JRadioButton("as displayed");
//        customSizeRadioButton = new JRadioButton("custom");
//        customSizeRadioButton.setSelected(true);
//        sizeButtonGroup = new ButtonGroup();
//        displayedSizeRadioButton.addItemListener(new
//                SaveChartImageDialog_displayedSizeRadioButton_itemAdapter(this));
//        sizeButtonGroup.add(displayedSizeRadioButton);
//        sizeButtonGroup.add(customSizeRadioButton);
        widthLabel = new JLabel("width");
        widthTextField = new JTextField(4);
        widthTextField.setText("" + imageWidth);
        heightLabel = new JLabel("height");
        heightTextField = new JTextField(4);
        heightTextField.setText("" + imageHeight);
        fileLabel = new JLabel("Filename");
        fileTextField = new JTextField(16);
        browseButton = new JButton("Browse...");
        browseButton.addActionListener(new
                SaveChartImageDialog_browseButton_actionAdapter(this));
        saveButton = new JButton("Save");
        cancelButton = new JButton("Cancel");
        saveButton.addActionListener(new
                SaveChartImageDialog_saveButton_actionAdapter(this));
        cancelButton.addActionListener(new
                SaveChartImageDialog_cancelButton_actionAdapter(this));

        SpringLayout layout = (SpringLayout) contentPane.getLayout();
        contentPane.add(imageSizeLabel);
        /*contentPane.add(displayedSizeRadioButton);
        layout.putConstraint(SpringLayout.WEST, displayedSizeRadioButton,
                             20,
                             SpringLayout.WEST, imageSizeLabel);
         * 
         */

        /*layout.putConstraint(SpringLayout.NORTH, displayedSizeRadioButton,
                             5,
                             SpringLayout.SOUTH, imageSizeLabel);
        contentPane.add(customSizeRadioButton);
        layout.putConstraint(SpringLayout.WEST, customSizeRadioButton, 20,
                             SpringLayout.WEST, imageSizeLabel);
        layout.putConstraint(SpringLayout.NORTH, customSizeRadioButton, 5,
                             SpringLayout.SOUTH, displayedSizeRadioButton);
         * 
         */
        

        contentPane.add(fileLabel);
        layout.putConstraint(SpringLayout.WEST, fileLabel,
                             0,
                             SpringLayout.WEST, imageSizeLabel);
        layout.putConstraint(SpringLayout.NORTH, fileLabel,
                             30,
                             SpringLayout.SOUTH, widthLabel);
        
        contentPane.add(widthLabel);
        layout.putConstraint(SpringLayout.EAST, widthLabel, 0,
                             SpringLayout.EAST, fileLabel);
        layout.putConstraint(SpringLayout.NORTH, widthLabel, 40,
                             SpringLayout.NORTH, imageSizeLabel);
        contentPane.add(widthTextField);
        layout.putConstraint(SpringLayout.WEST, widthTextField, 10,
                             SpringLayout.EAST, widthLabel);
        layout.putConstraint(SpringLayout.SOUTH, widthTextField, 0,
                             SpringLayout.SOUTH, widthLabel);
        contentPane.add(heightLabel);
        layout.putConstraint(SpringLayout.WEST, heightLabel, 10,
                             SpringLayout.EAST, widthTextField);
        layout.putConstraint(SpringLayout.SOUTH, heightLabel, 0,
                             SpringLayout.SOUTH, widthLabel);
        contentPane.add(heightTextField);
        layout.putConstraint(SpringLayout.WEST, heightTextField, 5,
                             SpringLayout.EAST, heightLabel);
        layout.putConstraint(SpringLayout.SOUTH, heightTextField, 0,
                             SpringLayout.SOUTH, heightLabel);

        contentPane.add(fileTextField);
        layout.putConstraint(SpringLayout.WEST, fileTextField, 10,
                             SpringLayout.EAST, fileLabel);
        layout.putConstraint(SpringLayout.SOUTH, fileTextField, 0,
                             SpringLayout.SOUTH, fileLabel);

        browseButton.setPreferredSize(new Dimension(fileTextField.getPreferredSize().width, fileTextField.getPreferredSize().height));
        contentPane.add(browseButton);
        layout.putConstraint(SpringLayout.WEST, browseButton, 0,
                             SpringLayout.WEST, fileTextField);
        layout.putConstraint(SpringLayout.NORTH, browseButton, 5,
                             SpringLayout.SOUTH, fileTextField);

        contentPane.add(saveButton);
        layout.putConstraint(SpringLayout.WEST, saveButton, 50,
                             SpringLayout.WEST, fileLabel);
        layout.putConstraint(SpringLayout.NORTH, saveButton, 150,
                             SpringLayout.NORTH, imageSizeLabel);
        contentPane.add(cancelButton);
        layout.putConstraint(SpringLayout.WEST, cancelButton, 20,
                             SpringLayout.EAST, saveButton);
        layout.putConstraint(SpringLayout.SOUTH, cancelButton, 0,
                             SpringLayout.SOUTH, saveButton);
    }

    /**
     * Handles the event fired when the user clicks save button.
     *
     * @param e the <code>ActionEvent</code> fired when the user
     *        clicks save button
     */
    public void saveButton_actionPerformed(ActionEvent e) {
        //if (customSizeRadioButton.isSelected()) {
            try {
                readWidthInput();
                readHeightInput();
            }
            catch (Exception ex) {
                return;
            }
        //}

        /**
         * user hit approve option; get the specified path for the file
         */
        String filePath = fileTextField.getText();

        try {
            saveImage((PanelExpressionChart)chartPanel, filePath, null, 
                    imageWidth, imageHeight, this.getOwner());
        }
        catch (Exception ex) {
            JOptionPane.showMessageDialog(this.getOwner(),
                                          "Unable to save image file",
                                          "Error saving image file",
                                          JOptionPane.INFORMATION_MESSAGE);
        }
        finally {
            this.dispose();
        }
    }
    
    public static void saveImage(PanelExpressionChart chartPanel, String filePath, 
            String extension, int imageWidth, int imageHeight, Component owner) throws IOException {
        if (filePath == null || filePath.equals("")){
            JOptionPane.showMessageDialog(owner,
                                          "No filename." +
                                          "\nYou must provide a filename.",
                                          "Filename error",
                                          JOptionPane.ERROR_MESSAGE);
            return;
        }
        if (extension == null) {
            extension = filePath.substring(filePath.length()-4,
                                              filePath.length());
        }
                
        File newFile = new File(filePath);
            if (newFile.exists()) {
                if (newFile.isDirectory()) {
                    JOptionPane.showMessageDialog(owner,
                                                  "Unable to save image." +
                                                  "\nThe specified filename " +
                                                  "corresponds to a directory.",
                                                  "Filename error",
                                                  JOptionPane.ERROR_MESSAGE);
                    return;
                }
                if (newFile.isFile()) {
                    int opt = JOptionPane.showOptionDialog(owner,
                            "File already exists." +
                            "\nOverwrite?",
                            "File exists",
                            JOptionPane.YES_NO_OPTION,
                            JOptionPane.QUESTION_MESSAGE,
                            null, null, JOptionPane.YES_OPTION);
                    if (opt == JOptionPane.NO_OPTION) {
                        /**
                         * the user does not want to overwrite the file
                         */
                        return;
                    }
                }
            }
            if (extension.equals(PNG)) {
                ChartUtilities.saveChartAsPNG(newFile,
                                              chartPanel.getChart(),
                                              imageWidth, imageHeight);
            } else
            if (extension.equals(SVG)){
                BiggestsIO.exportChartAsSVG(
                        newFile,
                        chartPanel.getChart(),
                        imageWidth, imageHeight);
            }
            else
            if (extension.equals(PDF)){
                BiggestsIO.exportChartAsPDF(newFile,
                               chartPanel.getChart(),
                               imageWidth, imageHeight,
                               new DefaultFontMapper());
            }
            else {
                JOptionPane.showMessageDialog(owner,
                                              "Invalid file extension." +
                                              "\nValid file extensions are: " + 
                                              ".png, .svg or .pdf.",
                                              "Filename error",
                                              JOptionPane.ERROR_MESSAGE);
                return;
            }
    }


    /**
     * Handles the event fired when the user clicks cancel button.
     *
     * @param e the <code>ActionEvent</code> fired when the user
     *        clicks save button
     */
    public void cancelButton_actionPerformed(ActionEvent e) {
        this.dispose();
    }

    /**
     * Reads the width input from the corresponding textfield.
     *
     * @throws Exception
     */
    public void readWidthInput() throws Exception {
        int width;
        try {
            width = Integer.parseInt(widthTextField.getText());
        }
        catch (NumberFormatException ex) {
            JOptionPane.showMessageDialog(this.getOwner(),
                    "Width must be an integer value.",
                    "Width must be integer",
                    JOptionPane.INFORMATION_MESSAGE);
            throw new Exception();
        }

        if (width <= 0) {
            JOptionPane.showMessageDialog(this.getOwner(),
                    "Width must be a positive integer value.",
                    "Width must be a positive integer",
                    JOptionPane.INFORMATION_MESSAGE);
            throw new Exception();
        }
        imageWidth = width;
    }

    /**
     * Reads the height input from the corresponding text field.
     *
     * @throws Exception
     */
    public void readHeightInput() throws Exception {
        int height;
        try {
            height = Integer.parseInt(heightTextField.getText());
        }
        catch (NumberFormatException ex) {
            JOptionPane.showMessageDialog(this.getOwner(),
                    "Height must be an integer value.",
                    "Height must be integer",
                    JOptionPane.INFORMATION_MESSAGE);
            throw new Exception();
        }

        if (height <= 0) {
            JOptionPane.showMessageDialog(this.getOwner(),
                    "Height must be a positive integer value.",
                    "Height must be a positive integer",
                    JOptionPane.INFORMATION_MESSAGE);
            throw new Exception();
        }
        imageHeight = height;
    }

    /**
     * Handles the event fired when the state of the displayed
     * size radio button changes.
     *
     * @param itemEvent the <code>ItemEvent</code> fired when the state
     *        of the displayed size radio button changes
     */
/*    public void displayedSizeRadioButton_itemStateChanged(ItemEvent itemEvent) {
        boolean enable = !displayedSizeRadioButton.isSelected();
        widthLabel.setEnabled(enable);
        widthTextField.setEnabled(enable);
        heightLabel.setEnabled(enable);
        heightTextField.setEnabled(enable);
    }
     * 
     */

    public void browseButton_actionPerformed(ActionEvent e) {
        String newPath = browsePath(this);
        if (newPath == null) return;
        fileTextField.setText(newPath);
    }
    
    public static String browsePath(Component component) {
        JFileChooser pathFileChooser = new JFileChooser();
        try {
            /**
             * initialize the file chooser with the intended file filters
             * (one for png files, and another for svg files)
             */
            pathFileChooser.addChoosableFileFilter(
                    new FileFilterExtension(".svg"));
            pathFileChooser.addChoosableFileFilter(
                    new FileFilterExtension(".pdf"));
            pathFileChooser.addChoosableFileFilter(
                    new FileFilterExtension(
                            ".png"));
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        /**
         * turn off accept all file filter
         */
        pathFileChooser.setAcceptAllFileFilterUsed(false);

        /**
         * show the save dialog to the user
         */
        int option = pathFileChooser.showSaveDialog(component);

        if (option == JFileChooser.APPROVE_OPTION) {
            /**
             * user hit approve option; get the specified path for the file
             */
            String newPath = pathFileChooser.getSelectedFile().getPath();
            newPath = ((FileFilterExtension) pathFileChooser.getFileFilter()).
                      checkAndAddFileExtension(newPath);
            return newPath;
        }
        return null;
    }
}


class SaveChartImageDialog_browseButton_actionAdapter implements ActionListener {
    private SaveChartImageDialog adaptee;
    SaveChartImageDialog_browseButton_actionAdapter(SaveChartImageDialog adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.browseButton_actionPerformed(e);
    }
}


/*class SaveChartImageDialog_displayedSizeRadioButton_itemAdapter implements
        ItemListener {
    private SaveChartImageDialog adaptee;
    SaveChartImageDialog_displayedSizeRadioButton_itemAdapter(
            SaveChartImageDialog adaptee) {
        this.adaptee = adaptee;
    }

    public void itemStateChanged(ItemEvent itemEvent) {
        adaptee.displayedSizeRadioButton_itemStateChanged(itemEvent);
    }
}
 * 
 */


class SaveChartImageDialog_cancelButton_actionAdapter implements ActionListener {
    private SaveChartImageDialog adaptee;
    SaveChartImageDialog_cancelButton_actionAdapter(SaveChartImageDialog adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.cancelButton_actionPerformed(e);
    }
}


class SaveChartImageDialog_saveButton_actionAdapter implements ActionListener {
    private SaveChartImageDialog adaptee;
    SaveChartImageDialog_saveButton_actionAdapter(SaveChartImageDialog adaptee) {
        this.adaptee = adaptee;
    }

    public void actionPerformed(ActionEvent e) {
        adaptee.saveButton_actionPerformed(e);
    }
}
