package biggests.utils;

import javax.swing.Icon;
import java.awt.Component;
import java.awt.Color;
import java.awt.Graphics;

/**
 * <p>Title: Arrow</p>
 *
 * <p>Description: Class that implements an Icon. Actually it is an
 *                 arrow representing the sorting status.</p>
 *
 * <p>Copyright: Copyright (c) 2008</p>
 *
 *
 * @author Joana P. Gonçalves
 * @version 1.0
 */
public class Arrow implements Icon {
    /**
     * The sorting status.
     */
    private boolean descending;

    /**
     * The size of the icon.
     */
    private int size;

    /**
     * The priority.
     */
    private int priority;

    /**
     * Creates a new arrow for the given sorting status, with the given
     * <code>size</code> and <code>priority</code>.
     *
     * @param descending <code>boolean</code> the sorting status
     * @param size <code>int</code> the size of the icon
     * @param priority <code>int</code> the priority
     */
    public Arrow(boolean descending, int size, int priority) {
        this.descending = descending;
        this.size = size;
        this.priority = priority;
    }

    /**
     * Paints the arrow.
     *
     * @param c the <code>Component</code> where the arrow will be painted
     * @param g the <code>Graphics</code> to draw the arrow
     * @param x <code>int</code> the x-coordinate
     * @param y <codeint</code> the y-coordinate
     */
    public void paintIcon(Component c, Graphics g, int x, int y) {
        Color color = c == null ? Color.GRAY : c.getBackground();
        // In a compound sort, make each succesive triangle 20%
        // smaller than the previous one.
        int dx = (int) (size / 2 * Math.pow(0.8, priority));
        int dy = descending ? dx : -dx;
        // Align icon (roughly) with font baseline.
        y = y + 5 * size / 6 + (descending ? -dy : 0);
        int shift = descending ? 1 : -1;
        g.translate(x, y);

        // Right diagonal.
        g.setColor(color.darker());
        g.drawLine(dx / 2, dy, 0, 0);
        g.drawLine(dx / 2, dy + shift, 0, shift);

        // Left diagonal.
        g.setColor(color.brighter());
        g.drawLine(dx / 2, dy, dx, 0);
        g.drawLine(dx / 2, dy + shift, dx, shift);

        // Horizontal line.
        if (descending) {
            g.setColor(color.darker().darker());
        } else {
            g.setColor(color.brighter().brighter());
        }
        g.drawLine(dx, 0, 0, 0);

        g.setColor(color);
        g.translate( -x, -y);
    }

    /**
     * Returns the width of the arrow icon.
     *
     * @return int the width of the arrow
     */
    public int getIconWidth() {
        return size;
    }

    /**
     * Returns the height of the arrow icon.
     *
     * @return int the height of the arrow
     */
    public int getIconHeight() {
        return size;
    }
}
